@@ -1,4 +1,11 @@
-totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.random = 0.1, alpha.fixed = 0.05, reduce.fixed = TRUE, reduce.random = TRUE, lsmeans.calc = TRUE, difflsmeans.calc=TRUE,  isTotal=FALSE, isAnova=FALSE, isRand=FALSE, isLSMEANS=FALSE, isDiffLSMEANS=FALSE, isTtest=FALSE, test.effs=NULL, method.grad="simple")
+totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, 
+                                  alpha.random = 0.1, alpha.fixed = 0.05, 
+                                  reduce.fixed = TRUE, reduce.random = TRUE, 
+                                  fixed.calc = TRUE, lsmeans.calc = TRUE, 
+                                  difflsmeans.calc=TRUE,  isTotal=FALSE, 
+                                  isAnova=FALSE, isRand=FALSE, isLSMEANS=FALSE, 
+                                  isDiffLSMEANS=FALSE, isTtest=FALSE, 
+                                  test.effs=NULL, method.grad="simple")
 {
   #errors in specifying the parameters
   if(!isRand && !(ddf %in% c("Satterthwaite","Kenward-Roger")))
@@ -17,8 +24,8 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
   
   #update contrasts
   mm <- model.matrix(model)
-  l <- attr(mm,"contrasts")
-  contr <- l
+  l.lmerTest.private.contrast<- attr(mm,"contrasts")
+  contr <- l.lmerTest.private.contrast
   ### change contrasts for F tests calculations
   #list of contrasts for factors
   if( isAnova || isTotal )
@@ -26,17 +33,17 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
     if( length(which(unlist(contr)!="contr.SAS")) > 0 )
     {
       names.facs <- names(contr)
-      l <- as.list(rep("contr.SAS",length(names.facs)))
-      names(l) <- names(contr)
+      l.lmerTest.private.contrast <- as.list(rep("contr.SAS",length(names.facs)))
+      names(l.lmerTest.private.contrast) <- names(contr)
       #warning(" \nmodel has been refitted with contrasts=contr.SAS \n")
       #model<-update(model,.~., data=data, contrasts=l)
-	    model <- updateModel(model, .~., getREML(model), l) 
+	    model <- updateModel(model, .~., getREML(model), l.lmerTest.private.contrast) 
     }    
   }
   else
   {
     #update model to mer class
-	model <- updateModel(model, .~., getREML(model), l)
+	model <- updateModel(model, .~., getREML(model), l.lmerTest.private.contrast)
   }
   
   
@@ -70,7 +77,7 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
 #       else
 #       {
 #         warning("\n model has been refitted with REML=TRUE \n")
-#         updateModel(model, .~., reml=TRUE, l)
+#         updateModel(model, .~., reml=TRUE, l.lmerTest.private.contrast)
 #       }
 #   }
 #   
@@ -82,7 +89,7 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
   
   
     
-  model <- updateModel(model, mf.final, getREML(model), l)
+  model <- updateModel(model, mf.final, getREML(model), l.lmerTest.private.contrast)
   
    
   
@@ -93,12 +100,12 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
   result$corr.intsl <- checkCorr(model)  
   
   
- 
-  if(isRand || isTotal)
-  {
-    result.rand <- elimZeroVarOrCorr(model, data, l)
-    model <- result.rand$model      
-  }    
+ ## removed as elimRandEffs may do the same as elimZeroVarOrCorr, so no need
+  #if(isRand || isTotal)
+ # {
+ #   result.rand <- elimZeroVarOrCorr(model, data, l.lmerTest.private.contrast)
+ #   model <- result.rand$model      
+ # }    
   
   
   #save results for fixed effects for model with only fixed effects
@@ -117,7 +124,8 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
   {
     if(isRand)
       reduce.random <- FALSE
-    result.rand <- elimRandEffs(model, data, alpha.random, reduce.random, l)  
+    result.rand <- elimRandEffs(model, data, alpha.random, reduce.random, 
+                                l.lmerTest.private.contrast)  
    
     model <- result.rand$model
     #convert rand table to data frame
@@ -127,8 +135,11 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
       rt$elim.num <- as.integer(rt$elim.num)
     
     result$rand.table <- rt
-    if(isRand)
+    if(isRand || !fixed.calc){
+      result$model <- model
       return(result)
+    }
+      
   }
   
   
@@ -158,7 +169,8 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
         if(isLSMEANS || isDiffLSMEANS)
         {
           lsmeans.summ <-  matrix(ncol=7,nrow=0)
-          colnames(lsmeans.summ) <- c("Estimate","Standard Error", "DF", "t-value", "Lower CI", "Upper CI", "p-value")
+          colnames(lsmeans.summ) <- c("Estimate", "Standard Error", "DF", 
+                                      "t-value", "Lower CI", "Upper CI", "p-value")
           lsmeans.summ <- as.data.frame(lsmeans.summ)
           if(isLSMEANS)
             result$lsmeans.table <- lsmeans.summ
@@ -177,8 +189,10 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
           rho$A <- 2*solve(h)
           #rho$A <- 2*ginv(h)
           
-          tsummary <- calculateTtest(rho, diag(rep(1,length(rho$fixEffs))),length(rho$fixEffs), method.grad)
-          result$ttest <- list(df=tsummary[,"df"], tvalue=tsummary[,"t value"], tpvalue=tsummary[,"p-value"])
+          tsummary <- calculateTtest(rho, diag(rep(1,length(rho$fixEffs))),
+                                     length(rho$fixEffs), method.grad)
+          result$ttest <- list(df=tsummary[,"df"], tvalue=tsummary[,"t value"], 
+                               tpvalue=tsummary[,"p-value"])
         }
         result$model <- model
         result$anova.table <- anova(model, ddf="lme4")
@@ -241,13 +255,17 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
     {
       if(isLSMEANS)
       {
-        lsmeans.tab <- calcLSMEANS(model, data, rho, alpha.fixed, test.effs=test.effs, method.grad=method.grad, lsmeansORdiff=TRUE, l)
+        lsmeans.tab <- calcLSMEANS(model, data, rho, alpha.fixed, 
+                                   test.effs=test.effs, method.grad=method.grad,
+                                   lsmeansORdiff=TRUE, l.lmerTest.private.contrast)
         result$lsmeans.table <- lsmeans.tab$summ.data
         result$diffs.lsmeans.table <- NULL
       }
       if(isDiffLSMEANS)
       {
-        lsmeans.tab <- calcLSMEANS(model, data, rho, alpha.fixed, test.effs=test.effs, method.grad=method.grad, lsmeansORdiff=FALSE, l)
+        lsmeans.tab <- calcLSMEANS(model, data, rho, alpha.fixed, 
+                                   test.effs=test.effs, method.grad=method.grad,
+                                   lsmeansORdiff=FALSE, l.lmerTest.private.contrast)
         result$diffs.lsmeans.table <- lsmeans.tab$summ.data
         result$lsmeans.table <- NULL
       }
@@ -311,7 +329,9 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
     }
  
     #calculate ss F value ddf and p value for each term 
-    resultFpvalueSS <- lapply(test.terms, calcFpvalueMAIN, L=L, X.design=X.design, fullCoefs=fullCoefs, model=model, rho=rho, ddf=ddf, method.grad=method.grad, type=type)  
+    resultFpvalueSS <- lapply(test.terms, calcFpvalueMAIN, L=L, X.design=X.design,
+                              fullCoefs=fullCoefs, model=model, rho=rho, ddf=ddf,
+                              method.grad=method.grad, type=type)  
     #fill anova table
     anova.table <- fillAnovaTable(resultFpvalueSS,  anova.table)
     
@@ -320,14 +340,16 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
       break     
     else
     {
-      resNSelim <- elimNSFixedTerm(model, anova.table, data, alpha.fixed, elim.num, l)
+      resNSelim <- elimNSFixedTerm(model, anova.table, data, alpha.fixed, elim.num,
+                                   l.lmerTest.private.contrast)
       if(is.null(resNSelim))
         break
       else
       {
         model <- resNSelim$model
         mf.final <- update.formula(formula(model),formula(model))
-        model <- updateModel(model, mf.final, getREML(model), l)        
+        model <- updateModel(model, mf.final, getREML(model), 
+                             l.lmerTest.private.contrast)        
         anova.table <- updateAnovaTable(resNSelim)
         elim.num <- elim.num+1
        
@@ -352,7 +374,9 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
   #if in step function least squares means of diffs of LSMEANS are required
   if(lsmeans.calc)
   {
-    lsmeans.tab <- calcLSMEANS(model, data, rho, alpha.fixed, test.effs=test.effs, method.grad=method.grad, lsmeansORdiff=TRUE, l)
+    lsmeans.tab <- calcLSMEANS(model, data, rho, alpha.fixed, test.effs=test.effs,
+                               method.grad=method.grad, lsmeansORdiff=TRUE, 
+                               l.lmerTest.private.contrast)
     result$lsmeans.table <- lsmeans.tab$summ.data
   }
   else
@@ -361,7 +385,9 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
   }
   if(difflsmeans.calc)
   {
-    lsmeans.tab <- calcLSMEANS(model, data, rho, alpha.fixed, test.effs=test.effs, method.grad=method.grad, lsmeansORdiff=FALSE, l)
+    lsmeans.tab <- calcLSMEANS(model, data, rho, alpha.fixed, test.effs=test.effs, 
+                               method.grad=method.grad, lsmeansORdiff=FALSE, 
+                               l.lmerTest.private.contrast)
     result$diffs.lsmeans.table <- lsmeans.tab$summ.data
   }
   else
@@ -369,8 +395,10 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
     result$diffs.lsmeans.table <- NULL
   }
   
-  tsummary <- calculateTtest(rho, diag(rep(1,length(rho$fixEffs))), length(rho$fixEffs), method.grad)
-  result$ttest <- list(df=tsummary[,"df"], tvalue=tsummary[,"t value"], tpvalue=tsummary[,"p-value"])
+  tsummary <- calculateTtest(rho, diag(rep(1,length(rho$fixEffs))), 
+                             length(rho$fixEffs), method.grad)
+  result$ttest <- list(df=tsummary[,"df"], tvalue=tsummary[,"t value"], 
+                       tpvalue=tsummary[,"p-value"])
   
   #format anova.table and random.table according to elim.num column
   result$anova.table <- formatElimNumTable(result$anova.table) 
@@ -392,14 +420,39 @@ totalAnovaRandLsmeans <- function(model, ddf="Satterthwaite", type = 3, alpha.ra
 }
 
 
-
-step <- function(model, ddf="Satterthwaite", type=3, alpha.random = 0.1, alpha.fixed = 0.05, reduce.fixed = TRUE, reduce.random = TRUE, lsmeans.calc=TRUE, difflsmeans.calc=TRUE, test.effs=NULL, method.grad="simple", ...)
+step <- function(model, ddf="Satterthwaite", type=3, alpha.random = 0.1, alpha.fixed = 0.05, reduce.fixed = TRUE, reduce.random = TRUE, fixed.calc=TRUE ,lsmeans.calc=TRUE, difflsmeans.calc=TRUE, test.effs=NULL, method.grad="simple", ...)
 {  
-  result <- totalAnovaRandLsmeans(model=model, ddf=ddf , type=type,  alpha.random=alpha.random, alpha.fixed=alpha.fixed, reduce.fixed=reduce.fixed, reduce.random=reduce.random, lsmeans.calc=lsmeans.calc, difflsmeans.calc=difflsmeans.calc, isTotal=TRUE, isTtest=FALSE, test.effs=test.effs, method.grad=method.grad)
+  if(!inherits(model, "lmerMod"))
+    stop("The model is not linear mixed effects model")
+
+  result <- totalAnovaRandLsmeans(model = model, ddf = ddf , type = type,  
+                                  alpha.random = alpha.random, 
+                                  alpha.fixed = alpha.fixed,
+                                  reduce.fixed = reduce.fixed, 
+                                  reduce.random = reduce.random,
+                                  fixed.calc = fixed.calc, 
+                                  lsmeans.calc = lsmeans.calc,
+                                  difflsmeans.calc = difflsmeans.calc, 
+                                  isTotal = TRUE, 
+                                  isTtest = FALSE, test.effs = test.effs, 
+                                  method.grad = method.grad)
   class(result) <- "step"
   result
 }
 
+#step.merModLmerTest <- function(object, scope, scale = 0,
+#direction = c("both", "backward", "forward"),
+#trace = 1, keep = NULL, steps = 1000, k = 2, ...)
+  #function(model, ddf="Satterthwaite", type=3, alpha.random = 0.1, alpha.fixed = 0.05, reduce.fixed = TRUE, reduce.random = TRUE, lsmeans.calc=TRUE, difflsmeans.calc=TRUE, test.effs=NULL, method.grad="simple", ...)
+#{  
+#  if(!inherits(object, "lmerMod"))
+#    stop("The model is not linear mixed effects model")
+ # result <- totalAnovaRandLsmeans(model=model, ddf=ddf , type=type,  alpha.random=alpha.random, alpha.fixed=alpha.fixed, reduce.fixed=reduce.fixed, reduce.random=reduce.random, lsmeans.calc=lsmeans.calc, difflsmeans.calc=difflsmeans.calc, isTotal=TRUE, isTtest=FALSE, test.effs=test.effs, method.grad=method.grad)
+#  result <- 2
+#  class(result) <- "step.merModLmerTest"
+#  result
+#}
+
 ### UNUSED function
 #totalAnalysis.formula <- function(formula, data, ...)
 #{
@@ -409,65 +462,77 @@ step <- function(model, ddf="Satterthwaite", type=3, alpha.random = 0.1, alpha.f
 #  resAnalysis
 #}
 
-print.step <- function(x, ...)
-{
-  
-  if(!is.null(x$rand.table))
-  {
-    cat("\nRandom effects:\n") 
-    x$rand.table[,"p.value"] <- format.pval(x$rand.table[,"p.value"], digits=4, eps=1e-7)
-    x$rand.table[,"Chi.sq"] <- round(x$rand.table[,"Chi.sq"],2)
-    print(x$rand.table)
-    #printCoefmat(x$rand.table, digits=3 , dig.tst=1  ,tst.ind=which(colnames(x$rand.table)=="Chi.DF"), P.values=TRUE, has.Pvalue=TRUE, na.print = "KEEP")
-  }
-  
-  if(nrow(x$anova.table) != 0)
-  {
-    if(class(x$model) == "lm" | class(x$model) == "gls")
-    {
-      cat("\nFixed effects:\n")
-      print(x$anova.table)
-      cat("\nLeast squares means:\n")
-      print(x$lsmeans.table) 
-      cat("\nFinal model:\n")
-      print(x$model)
-      return()
-    }
-    else
-    {
-      cat("\nFixed effects:\n")
-      x$anova.table[,"Pr(>F)"] <- format.pval(x$anova.table[,"Pr(>F)"], digits=4, eps=1e-7)
-      x$anova.table[,c("Sum Sq","Mean Sq", "F.value")] <- round(x$anova.table[,c("Sum Sq","Mean Sq", "F.value")],4)
-      x$anova.table[,"DenDF"] <- round(x$anova.table[,"DenDF"],2)
-      print(x$anova.table) 
-       #printCoefmat(x$anova.table, dig.tst=3, tst.ind=3, cs.ind=3, digits=3 ,P.values = TRUE, has.Pvalue=TRUE)
-      if(!is.null(x$lsmeans.table))
-      {
-        cat("\nLeast squares means:\n")
-        printCoefmat(x$lsmeans.table, dig.tst=3 ,tst.ind=c(1:(which(colnames(x$lsmeans.table)=="Estimate")-1),which(colnames(x$lsmeans.table)=="DF")), digits=3 ,P.values = TRUE, has.Pvalue=TRUE)
-      }
-      if(!is.null(x$diffs.lsmeans.table))
-      {
-        cat("\n Differences of LSMEANS:\n")
-        printCoefmat(x$diffs.lsmeans.table, dig.tst=1  ,tst.ind=c(1:(which(colnames(x$diffs.lsmeans.table)=="Estimate")-1),which(colnames(x$diffs.lsmeans.table)=="DF")), digits=3 ,P.values = TRUE, has.Pvalue=TRUE)
-      }
-      
-    }    
-  }
-  else
-    print(x$anova.table)
-  cat("\nFinal model:\n")
-  print(x$model@call) 
-}
-
-
-plot.step <- function(x, ...)
-{
-  if(!is.null(x$lsmeans.table) && nrow(x$lsmeans.table)>0)
-    plotLSMEANS(x$lsmeans.table, x$response, "LSMEANS")     
-  if(!is.null(x$diffs.lsmeans.table) && nrow(x$diffs.lsmeans.table)>0)
-    plotLSMEANS(x$diffs.lsmeans.table, x$response, "DIFF of LSMEANS")
-}
+ print.step <- function(x, ...)
+ {
+   
+   if(!is.null(x$rand.table))
+   {
+     cat("\nRandom effects:\n") 
+     x$rand.table[,"p.value"] <- format.pval(x$rand.table[,"p.value"], digits=4, 
+                                             eps=1e-7)
+     x$rand.table[,"Chi.sq"] <- round(x$rand.table[,"Chi.sq"], 2)
+     print(x$rand.table)     
+   } 
+   if(is.null(x$anova.table)){
+     
+   }else{
+     if(nrow(x$anova.table) != 0)
+     {
+       if(class(x$model) == "lm" | class(x$model) == "gls")
+       {
+         cat("\nFixed effects:\n")
+         print(x$anova.table)
+         cat("\nLeast squares means:\n")
+         print(x$lsmeans.table) 
+         cat("\nFinal model:\n")
+         print(x$model)
+         return()
+       }
+       else
+       {
+         cat("\nFixed effects:\n")
+         x$anova.table[,"Pr(>F)"] <- format.pval(x$anova.table[,"Pr(>F)"], 
+                                                 digits=4, eps=1e-7)
+         x$anova.table[,c("Sum Sq","Mean Sq", "F.value")] <- 
+           round(x$anova.table[,c("Sum Sq","Mean Sq", "F.value")],4)
+         x$anova.table[,"DenDF"] <- round(x$anova.table[,"DenDF"],2)
+         print(x$anova.table)          
+         if(!is.null(x$lsmeans.table))
+         {
+           cat("\nLeast squares means:\n")
+           printCoefmat(x$lsmeans.table, dig.tst=3 ,
+                        tst.ind=c(1:(which(colnames(x$lsmeans.table)=="Estimate")-1),
+                                  which(colnames(x$lsmeans.table)=="DF")), 
+                        digits=3 , P.values = TRUE, has.Pvalue=TRUE)
+         }
+         if(!is.null(x$diffs.lsmeans.table))
+         {
+           cat("\n Differences of LSMEANS:\n")
+           printCoefmat(x$diffs.lsmeans.table, dig.tst=1  ,
+                        tst.ind=c(1:(which(colnames(x$diffs.lsmeans.table)==
+                                             "Estimate")-1),
+                                  which(colnames(x$diffs.lsmeans.table)=="DF")),
+                        digits=3 , P.values = TRUE, has.Pvalue=TRUE)
+         }
+         
+       }    
+     }
+     else
+       print(x$anova.table)
+   }
+   
+   cat("\nFinal model:\n")
+   print(x$model@call) 
+ }
+ 
+ 
+ plot.step <- function(x, ...)
+ {
+   if(!is.null(x$lsmeans.table) && nrow(x$lsmeans.table)>0)
+     plotLSMEANS(x$lsmeans.table, x$response, "LSMEANS")     
+   if(!is.null(x$diffs.lsmeans.table) && nrow(x$diffs.lsmeans.table)>0)
+     plotLSMEANS(x$diffs.lsmeans.table, x$response, "DIFF of LSMEANS")
+ }
 
 # lmer <-
 #   function(formula, data, family = NULL, REML = TRUE,
@@ -508,26 +573,31 @@ plot.step <- function(x, ...)
 
 
 setMethod("anova", signature(object="merModLmerTest"),
-          function(object,..., ddf="Satterthwaite", type=3, method.grad="simple")  
+          function(object, ..., ddf="Satterthwaite", type=3, method.grad="simple")  
           {
             
             mCall <- match.call(expand.dots = TRUE)
             dots <- list(...)
             modp <- if (length(dots))
-              sapply(dots, is, "merModLmerTest") | sapply(dots, is, "merMod") | sapply(dots, is, "lm") else logical(0)
+              sapply(dots, is, "merModLmerTest") | sapply(dots, is, "merMod") | 
+              sapply(dots, is, "lm") else logical(0)
             if (any(modp)) {
               return(callNextMethod())
             }
             else
             {
               cnm <- callNextMethod()
-              if(!is.null(ddf)&& ddf=="lme4") 
+              if(!is.null(ddf) &&  ddf=="lme4") 
                 return(cnm)              
-             {
-                  table <- cnm
+              {
+                  table <- cnm          
                   
-                  
-                  an.table <- tryCatch({totalAnovaRandLsmeans(model=object, ddf=ddf, type=type, isAnova=TRUE, reduce.random=FALSE, reduce.fixed=FALSE, method.grad=method.grad)$anova.table}, error = function(e) { NULL })
+                  an.table <- tryCatch({totalAnovaRandLsmeans(model=object, 
+                                                              ddf=ddf, type=type,
+                                                              isAnova=TRUE, 
+                                                              reduce.random=FALSE,
+                                                              reduce.fixed=FALSE, 
+                                                              method.grad=method.grad)$anova.table}, error = function(e) { NULL })
                   if(!is.null(an.table))
                   {
                     table <- an.table
@@ -545,6 +615,8 @@ setMethod("anova", signature(object="merModLmerTest"),
                     
                     attr(table, "heading") <- paste("Analysis of Variance Table of type", type ," with ", ddf, "\napproximation for degrees of freedom")
                   }
+                  else
+                    message("anova from lme4 is returned\nsome computational error has occurred in lmerTest")
                   
                   
                   
@@ -565,9 +637,11 @@ setMethod("summary", signature(object = "merModLmerTest"),
             else
             {
               tsum <- tryCatch( {totalAnovaRandLsmeans(model=object, ddf="Satterthwaite", isTtest=TRUE)$ttest}, error = function(e) { NULL })
-              coefs.satt <- cbind(cl$coefficients[,1:2, drop=FALSE], tsum$df, tsum$tvalue, tsum$tpvalue)
-               # t.pval <- tryCatch( {totalAnovaRandLsmeans(model=object, ddf="Satterthwaite", isTtest=TRUE)$ttest$tpvalue}, error = function(e) { NULL })
-               # coefs.satt <- cbind(cl$coefficients, t.pval) 
+              if(is.null(tsum)){
+                message("summary from lme4 is returned\nsome computational error has occurred in lmerTest")
+                return(cl)
+              }
+              coefs.satt <- cbind(cl$coefficients[,1:2, drop=FALSE], tsum$df, tsum$tvalue, tsum$tpvalue)               
                 cl$coefficients <- coefs.satt
                 colnames(cl$coefficients)[3:5] <- c("df","t value","Pr(>|t|)")              
             }   
@@ -580,6 +654,8 @@ setMethod("summary", signature(object = "merModLmerTest"),
 #randTAB.default<-function(model, data, ...)
 rand <- function(model, ...)
 {
+  if(!inherits(model, "lmerMod"))
+    stop("The model is not linear mixed effects model")
   result <- totalAnovaRandLsmeans(model=model, isRand=TRUE, reduce.random=FALSE)  
   res <- list(rand.table=result$rand.table, isCorr = result$corr.intsl)
   class(res) <- "rand"
@@ -591,7 +667,10 @@ print.rand <- function(x, ...)
   
   cat("Analysis of Random effects Table:\n")
   if(!is.null(x$rand.table))
-    printCoefmat(x$rand.table, digits=3 , dig.tst=1  ,tst.ind=c(which(colnames(x$rand.table)=="Chi.DF"),which(colnames(x$rand.table)=="elim.num")), P.values=TRUE, has.Pvalue=TRUE)        
+    printCoefmat(x$rand.table, digits=3 , dig.tst=1  ,
+                 tst.ind=c(which(colnames(x$rand.table)=="Chi.DF"),
+                           which(colnames(x$rand.table)=="elim.num")), 
+                 P.values=TRUE, has.Pvalue=TRUE)        
 }
 
 
@@ -600,7 +679,12 @@ print.rand <- function(x, ...)
 
 lsmeans <- function(model, test.effs=NULL, method.grad="simple", ...)
 {
-  result <- totalAnovaRandLsmeans(model=model, ddf="Satterthwaite", isLSMEANS=TRUE, test.effs=test.effs, reduce.random=FALSE, reduce.fixed=FALSE, method.grad=method.grad)  
+  if(!inherits(model, "lmerMod"))
+    stop("The model is not linear mixed effects model")
+  result <- totalAnovaRandLsmeans(model=model, ddf="Satterthwaite", 
+                                  isLSMEANS=TRUE, test.effs=test.effs, 
+                                  reduce.random=FALSE, reduce.fixed=FALSE, 
+                                  method.grad=method.grad)  
   res <- list(lsmeans.table=result$lsmeans.table, response=result$response)
   class(res) <- "lsmeans"
   res 
@@ -610,7 +694,10 @@ print.lsmeans <- function(x, ...)
 {
   
   cat("Least Squares Means table:\n")
-  printCoefmat(data.matrix(x$lsmeans.table), dig.tst=1, tst.ind=c(1:(which(colnames(x$lsmeans.table)=="Estimate")-1),which(colnames(x$lsmeans.table)=="DF")), digits=3 , P.values=TRUE, has.Pvalue=TRUE)       
+  printCoefmat(data.matrix(x$lsmeans.table), dig.tst=1, 
+               tst.ind=c(1:(which(colnames(x$lsmeans.table)=="Estimate")-1),
+                         which(colnames(x$lsmeans.table)=="DF")), digits=3 , 
+               P.values=TRUE, has.Pvalue=TRUE)       
 }
 
 plot.lsmeans <- function(x, ...)
@@ -623,8 +710,14 @@ plot.lsmeans <- function(x, ...)
 
 difflsmeans <- function(model, test.effs=NULL, method.grad="simple", ...)
 {
-  result <- totalAnovaRandLsmeans(model=model, ddf="Satterthwaite", isDiffLSMEANS=TRUE, test.effs=test.effs, reduce.random=FALSE, reduce.fixed=FALSE, method.grad=method.grad)  
-  res <- list(diffs.lsmeans.table=result$diffs.lsmeans.table, response=result$response)
+  if(!inherits(model, "lmerMod"))
+    stop("The model is not linear mixed effects model")
+  result <- totalAnovaRandLsmeans(model=model, ddf="Satterthwaite", 
+                                  isDiffLSMEANS=TRUE, test.effs=test.effs, 
+                                  reduce.random=FALSE, reduce.fixed=FALSE, 
+                                  method.grad=method.grad)  
+  res <- list(diffs.lsmeans.table=result$diffs.lsmeans.table, 
+              response=result$response)
   class(res) <- "difflsmeans"
   res 
 }
@@ -633,7 +726,10 @@ print.difflsmeans <- function(x, ...)
 {
   
   cat("Differences of LSMEANS:\n")
-  printCoefmat(data.matrix(x$diffs.lsmeans.table), dig.tst=1, tst.ind=c(1:(which(colnames(x$diffs.lsmeans.table)=="Estimate")-1),which(colnames(x$diffs.lsmeans.table)=="DF")), digits=3 , P.values=TRUE, has.Pvalue=TRUE)
+  printCoefmat(data.matrix(x$diffs.lsmeans.table), dig.tst=1, 
+               tst.ind=c(1:(which(colnames(x$diffs.lsmeans.table)=="Estimate")-1),
+                         which(colnames(x$diffs.lsmeans.table)=="DF")), digits=3 ,
+               P.values=TRUE, has.Pvalue=TRUE)
   
 }
 
